`timescale 1ns / 1ps
`default_nettype none
`define buswidth 16
`define MAXADDR 12287

module simple_logiana(
	input clk_in,
	input [`buswidth-1:0] din
);

wire shift, jtdi, drck1, drck2, capture, jtdo1, jtdo2;
reg [14:0] addrb;
wire [7:0] doutb;
reg [2:0] bytecount, bytecount2;

reg [13:0] addra;
reg [7:0] memreg;
reg overlap;

reg [7:0] config_reg;
reg [7:0] trig_info;
reg [`buswidth-1:0] din_reg0, din_reg1, din_reg2;
reg [7:0] clk_divide;
reg [7:0] clk_count;
reg [7:0] tdibuf, tdobuf;
reg [7:0] dummy_reg;
reg [13:0] trig_count;
reg [13:0] trig_addr;
reg [2:0] regaddr;
reg addr_stop;
reg trig_det, trig_val0, trig_en;

reg [13:0] test_count;
reg acq_en;

wire [7:0] status = {5'b00000, overlap, addr_stop, trig_det};
wire [3:0] trig_ch = trig_info[3:0];
wire [2:0] trig_pos = trig_info[6:4];	// 12.5%(0), 25%(1) ... 87.5%(6)
wire trig_pol = trig_info[7];	// 0: falling edge, 1: rising edge
reg [13:0] trig_stop;

wire clk, clk2x, clkfx;
wire reset_sig = config_reg[0];
wire acq_start = config_reg[1];
wire clk_sel   = config_reg[2];	// 50MHz or 40MHz select
wire test_mode = config_reg[3];
wire save_mode = config_reg[4];
wire addr_count_en = acq_en && acq_start && !addr_stop;

mem16bit ram_inst (
	.clka(acq_en), // input clka
	.wea(acq_start), // input [0 : 0] wea
	.addra(addra), // input [13 : 0] addra
	.dina(din_reg2), // input [15 : 0] dina
	.clkb(drck1), // input clkb
	.addrb(addrb), // input [14 : 0] addrb
	.doutb(doutb) // output [7 : 0] doutb
);

dcm100M dcm_inst (
	.CLKIN_IN(clk_in), 
	.RST_IN(1'b0), 
	.CLKFX_OUT(clkfx), 
	.CLKIN_IBUFG_OUT(), 
	.CLK0_OUT(), 
	.CLK2X_OUT(clk2x), 
	.LOCKED_OUT()
);

BUFGMUX clkmux_inst (.I0(clkfx), .I1(clk2x), .S(clk_sel), .O(clk));

BSCAN_SPARTAN3 jtag_inst (
	.UPDATE(), .SHIFT(shift), .RESET(),
	.TDI(jtdi), .SEL1(), .SEL2(),
	.DRCK1(drck1), .DRCK2(drck2), .CAPTURE(capture),
	.TDO1(jtdo1), .TDO2(jtdo2)
);

always @(posedge clk) begin
	if(reset_sig) begin
		din_reg0 <= 0;
		din_reg1 <= 0;
		din_reg2 <= 0;
		test_count <= 0;
	end
	else if(test_mode) begin
		if(acq_en) begin
			test_count <= test_count + 1;
			if(test_count > 256) begin
				din_reg1 <= {test_count[4:3],test_count};
				din_reg2 <= din_reg1;
			end
		end
	end
	else if(acq_en) begin
		din_reg0 <= din;
		din_reg1 <= din_reg0;
		din_reg2 <= din_reg1;
	end
end

always @(posedge clk) begin
	if(reset_sig) begin
		addra <= 0;
		overlap <= 0;
		trig_en <= 0;
	end
	else if(addr_count_en) begin
		addra <= addra + 1;
		if(addra == 3) begin
			trig_en <= 1;
		end
		else if(addra == `MAXADDR) begin
			addra <= 0;
			overlap <= 1;
		end
	end
end	

always @(posedge clk) begin
	if(reset_sig) begin
		trig_det <= 0;
		trig_val0 <= 0;
		trig_addr <= 0;
	end
	else if(!trig_det) begin
		// use trig_en to ignore first 3 data
		if(din_reg2[trig_ch]==trig_pol && trig_val0!=trig_pol && trig_en) begin
			trig_det <= 1;
			trig_addr <= addra;
		end
		trig_val0 <= din_reg2[trig_ch];
	end
end

always @* begin
	case(trig_pos)
		3'b000: trig_stop = 14'h2a00;
		3'b001: trig_stop = 14'h2400;
		3'b010: trig_stop = 14'h1e00;
		3'b011: trig_stop = 14'h1800;
		3'b100: trig_stop = 14'h1200;
		3'b101: trig_stop = 14'h0c00;
		default: trig_stop = 14'h0600;
	endcase
end

always @(posedge clk) begin
	if(reset_sig) begin
		addr_stop <= 0;
		trig_count <= 0;
	end
	else if(trig_det) begin
//		if(trig_count >= {trig_pos, 13'h1ff0}) begin
		if(trig_count >= trig_stop) begin
			addr_stop <= 1;
		end
		else if(acq_en) begin
			trig_count <= trig_count + 1;
		end
	end
end

always @(posedge clk) begin	// clock divider
	if(clk_count == clk_divide) begin
		clk_count <= 0;
		acq_en <= 1'b1;
	end
	else begin
		clk_count <= clk_count + 1;
		acq_en <= 1'b0;
	end
end

// ----------------- BSCAN control (PC interface) ---------------------

always @(posedge drck1) begin
	if(capture) begin
		addrb <= 0;
		bytecount <= 0;
	end
	else if(shift) begin
		bytecount <= bytecount + 1;
		if(bytecount==7) begin
			addrb <= addrb + 1;
			memreg <= doutb;
		end
	end
end

always @(posedge drck2) begin
	if(capture) begin
		bytecount2 <= 0;
		regaddr <= 0;
		tdibuf <= 0;
	end
	else if(shift) begin
		tdibuf <= {jtdi, tdibuf[7:1]};
		bytecount2 <= bytecount2 + 1;
		if(bytecount2 == 7) begin
			regaddr <= regaddr + 1;
			case(regaddr)
				3'b000 : config_reg <= {jtdi, tdibuf[7:1]};
				3'b001 : clk_divide <= {jtdi, tdibuf[7:1]};
				3'b010 : trig_info <= {jtdi, tdibuf[7:1]};
				default : dummy_reg <= {jtdi, tdibuf[7:1]}; 
			endcase
		end
	end
end

always @(posedge drck2) begin
	if(capture) begin
		tdobuf <= status;
	end
	else if(shift) begin
		if(bytecount2 == 7) begin
			case(regaddr)
				3'b000 : tdobuf <= trig_addr[13:8];
				3'b001 : tdobuf <= trig_addr[7:0];
				3'b010 : tdobuf <= addra[13:8];
				3'b011 : tdobuf <= addra[7:0];
				3'b100 : tdobuf <= trig_count[13:8];
				3'b101 : tdobuf <= trig_count[7:0];
				3'b110 : tdobuf <= din[15:8];
				default : tdobuf <= din[7:0];
			endcase
		end
	end
end

assign jtdo1 = memreg[bytecount];
assign jtdo2 = tdobuf[bytecount2];

endmodule
